﻿Option Strict On
Option Explicit On

Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices

Public Class ShellItemImageFactory

#Region "API"
    <DllImport("Shell32.dll", EntryPoint:="SHCreateItemFromParsingName")>
    <PreserveSig> Private Shared Function SHCreateItemFromParsingName(<[In], MarshalAs(UnmanagedType.LPWStr)> pszPath As String,
                                                                      <[In]> pbc As IntPtr,
                                                                      <[In], MarshalAs(UnmanagedType.LPStruct)> riid As Guid,
                                                                      <Out, MarshalAs(UnmanagedType.Interface)> ByRef pUnk As Object) As Integer
    End Function

    <DllImport("Gdi32.dll", EntryPoint:="DeleteObject")>
    Private Shared Function DeleteObject(<[In]> hgdiobj As IntPtr) As <MarshalAs(UnmanagedType.Bool)> Boolean
    End Function
#End Region

#Region "Enum"
    Public Enum SIIGBF As Integer
        SIIGBF_RESIZETOFIT = &H0&
        SIIGBF_BIGGERSIZEOK = &H1&
        SIIGBF_MEMORYONLY = &H2&
        SIIGBF_ICONONLY = &H4&
        SIIGBF_THUMBNAILONLY = &H8&
        SIIGBF_INCACHEONLY = &H10&
        SIIGBF_CROPTOSQUARE = &H20&
        SIIGBF_WIDETHUMBNAILS = &H40&
        SIIGBF_ICONBACKGROUND = &H80&
        SIIGBF_SCALEUP = &H100&
    End Enum
#End Region

#Region "Const"
    Private Const S_OK As Integer = 0

    Private Const IID_IShellItemImageFactory As String = "bcc18b79-ba16-442f-80c4-8a59c30c463b"
#End Region

#Region "Public Functions"
    Public Function GetImageFromShellItem(ShellItem As String,
                                          ImageSize As Integer,
                                          Optional Flags As SIIGBF = SIIGBF.SIIGBF_RESIZETOFIT) As Bitmap

        Dim bmpOut As Bitmap = Nothing
        Dim oIShellItemImageFactory As Object = Nothing

        ' ein IShellItemImageFactory von der Datei oder Ordner erstellen
        If SHCreateItemFromParsingName(ShellItem, IntPtr.Zero,
                                       New Guid(IID_IShellItemImageFactory),
                                       oIShellItemImageFactory) = S_OK Then

            If oIShellItemImageFactory IsNot Nothing Then

                Dim hBitmap As IntPtr = IntPtr.Zero

                ' Handle auf ein Image holen
                If CType(oIShellItemImageFactory, IShellItemImageFactory).GetImage(New Size(ImageSize, ImageSize),
                                                                          Flags, hBitmap) = S_OK Then

                    If hBitmap <> IntPtr.Zero Then

                        ' Handle auf ein Image in ein .NET Bitmap konvertieren
                        bmpOut = GetBitmapFromHBitmap(hBitmap)

                        ' Handle auf das Image löschen
                        DeleteObject(hBitmap)

                    End If

                End If

                ' IShellItemImageFactory löschen
                Marshal.ReleaseComObject(oIShellItemImageFactory)

            End If

        End If

        Return bmpOut

    End Function

#End Region

#Region "Private Functions"
    Private Function GetBitmapFromHBitmap(hBitmap As IntPtr) As Bitmap
        Dim bmp As Bitmap = Bitmap.FromHbitmap(hBitmap)
        If Bitmap.GetPixelFormatSize(bmp.PixelFormat) < 32 Then Return bmp
        Dim bmpData As New BitmapData
        If IsAlphaBitmap(bmp, bmpData) Then Return GetlAlphaBitmapFromBitmapData(bmpData)
        Return bmp
    End Function

    Private Function GetlAlphaBitmapFromBitmapData(bmpData As BitmapData) As Bitmap
        Return New Bitmap(bmpData.Width, bmpData.Height, bmpData.Stride,
                          PixelFormat.Format32bppArgb, bmpData.Scan0)
    End Function

    Private Function IsAlphaBitmap(bmp As Bitmap, ByRef bmpData As BitmapData) As Boolean
        Dim bmBounds As New Rectangle(0, 0, bmp.Width, bmp.Height)
        bmpData = bmp.LockBits(bmBounds, ImageLockMode.ReadOnly, bmp.PixelFormat)
        Try
            For y As Integer = 0 To bmpData.Height - 1
                For x As Integer = 0 To bmpData.Width - 1
                    Dim pixelColor As Color = Color.FromArgb(Marshal.ReadInt32(bmpData.Scan0, (bmpData.Stride * y) + (4 * x)))
                    If pixelColor.A > 0 And pixelColor.A < 255 Then
                        Return True
                    End If
                Next
            Next
        Finally
            bmp.UnlockBits(bmpData)
        End Try
        Return False
    End Function
#End Region

#Region "Interface IShellItemImageFactory"
    <ComImport>
    <InterfaceType(ComInterfaceType.InterfaceIsIUnknown)>
    <Guid(IID_IShellItemImageFactory)>
    Private Interface IShellItemImageFactory
        <PreserveSig> Function GetImage(<[In]> size As Size,
                                        <[In]> flags As SIIGBF,
                                        <Out> ByRef phbm As IntPtr) As Integer
    End Interface
#End Region

End Class
